/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam;

import java.io.Serializable;

/**
 * Class Key represent a key in the ISAM file.
 */
public class Key<T extends Comparable<T>> implements Serializable, Comparable<Key<T>> {
    private static final long serialVersionUID = 1L;
    private int ix;
	private T value;
	/**
	 * Create instance of Key.
	 * @param ix key index
	 * @param value key value
	 */
	public Key(int ix, T value) {
		this.ix = ix;
		this.value = value;
	}
	public static <T extends Comparable<T>> Key<T> create(int ix, T value) {
		return new Key<T>(ix, value);
	}
	/**
	 * Get index.
	 * @return key index
	 */
	public int getIx() {
		return ix;
	}
	/**
	 * Get value.
	 * @return key value
	 */
	public T getValue() {
		return value;
	}
	/*
	 * Convert to String for logging purposes only.
	 */
	@Override
	public String toString() {
	    return "(" + ix + "," + value + ")"; 
	}
	/*
	 * Compare.
	 */
	@Override
	public int compareTo(Key<T> k) {
		return value.compareTo(k.value);
	}
}
