/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.remote;

/**
 * Interface IsamSource defines the wire protocol for remote ISAM sources.
 * This interface is not intended for use by applications.
 */
public interface IsamProtocol {
    /**
     * Object protocol version 1.
     */
    public static final int O_V1 = 1;
    /**
     * Byte protocol version 1.
     */
    public static final int B_V1 = 10;
	/**
	 * Read record from ISAM source.
	 */
	public static final int READ = 1;
    /**
     * Create record in ISAM source.
     */
    public static final int CREATE = 2;
    /**
     * Update record in ISAM source.
     */
    public static final int UPDATE = 3;
    /**
     * Delete record from ISAM source.
     */
    public static final int DELETE = 4;
	/**
	 * Close ISAM source.
	 */
	public static final int CLOSE = 9;
	/**
	 * Find all records greater than or equal key.
	 */
	public static final int FIND_GE = 10;
	/**
	 * Find all records greater than key.
	 */
	public static final int FIND_GT = 11;
	/**
	 * Find all records less than or equal key.
	 */
	public static final int FIND_LE = 12;
	/**
	 * Find all records less than key.
	 */
	public static final int FIND_LT = 13;
	/**
	 * Find all records.
	 */
	public static final int FIND_ALL = 14;
	/**
	 * Result: read record.
	 */
	public static final int RESULT_READ = 20;
	/**
	 * Result: return current record.
	 */
	public static final int RESULT_CURRENT = 21;
	/**
	 * Result: close.
	 */
	public static final int RESULT_CLOSE = 29;
    /**
     * OK.
     */
    public static final int OK = 30;
    /**
     * Error.
     */
    public static final int ERROR = 31;
}
