/*
 * Copyright 2023 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vbisam.test;

import static org.junit.Assert.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.isam.KeyInfoArrayCache;
import dk.vajhoej.isam.local.LocalIsamSource;
import dk.vajhoej.isam.test.Data;
import dk.vajhoej.isam.test.TestCommonIsamSource;
import dk.vajhoej.record.RecordException;

public class TestVBISAM {
	private static void exec(String cmd) throws IOException, InterruptedException {
        Process p = Runtime.getRuntime().exec(cmd);
        BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
        String line = br.readLine();
       	//System.out.println(line);
       	br.close();
       	//System.out.printf("exit = %d\n", p.waitFor());
	}
	private final static String FNM = "vbisam_test";
	private final static String CMD_CRE = "C:\\Code\\ISAM\\cre_vbisam_test.exe";
	private final static String CMD_DEL = "C:\\Code\\ISAM\\del_vbisam_test.exe";
	@Before
	public void setup() {
		try {
			exec(CMD_CRE);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	@After
	public void teardown() {
		try {
			exec(CMD_DEL);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
    @Test
    public void testCreateRead() {
		try {
			TestCommonIsamSource.testCreateRead(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testCreateReadKey0() {
		try {
			TestCommonIsamSource.testCreateReadKey0(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
	@Test
	public void testCreateReadMulti() {
		try {
			TestCommonIsamSource.testCreateReadMulti(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadPerf() {
		try {
			TestCommonIsamSource.testReadPerf(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testCreatePerf() {
		try {
			TestCommonIsamSource.testCreatePerf(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadStart() {
		try {
			TestCommonIsamSource.testReadStart(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGE() {
		try {
			TestCommonIsamSource.testReadGE(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGT() {
		try {
			TestCommonIsamSource.testReadGT(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	/*
	@Test
	public void testReadLE() {
		try {
			TestCommonIsamSource.testReadLE(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLT() {
		try {
			TestCommonIsamSource.testReadLT(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	*/
	@Test
	public void testError1() {
		try {
			TestCommonIsamSource.testError1(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
    @Test
    public void testError2() {
        try {
            TestCommonIsamSource.testError2(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testUpdate() {
        try {
            TestCommonIsamSource.testUpdate(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testDelete() {
        try {
            TestCommonIsamSource.testDelete(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testLogging() {
        try {
            TestCommonIsamSource.testLogging(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrency() {
        try {
            TestCommonIsamSource.testConcurrency(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false),
                                                 new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrencyHard() {
        try {
            TestCommonIsamSource.testConcurrencyHard(new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false),
                                                     new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
	@Test
	public void testAnalyzeCache() {
		try {
			IsamSource isam = new LocalIsamSource(FNM, "dk.vajhoej.vbisam.VBISAM", false);
			KeyInfoArrayCache.getInstance().reset();
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			isam.create(o);
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() < 0.001);
			for(int i = 1; i < 10; i++) {
				Data o2 = new Data();
				o2.setIv(258 + i);
				o2.setXv(123.456);
				o2.setSv("ABC     ");
				isam.create(o2);
			}
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() > 0.899);
			for(int i = 0; i < 10; i++) {
				Data o2 = isam.read(Data.class, new Key<Integer>(0, 258 + i));
				assertEquals("iv", 258 + i, o2.getIv());
			}
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() > 0.949);
			isam.close();
		} catch (IsamException e) {
			e.printStackTrace();
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
}

